import json
from collections import defaultdict

from django.http import HttpRequest, HttpResponse
from django.urls import path

from djangotools.common import response, errors
from djangotools.common.path import PointPath, ConstPath, AttributePath, UrlPath, ModelPath
from djangotools.common.route import RegisteredRoute, Post, Delete, Route, Get, Put
from djangotools.common import models



class AutoPathManager:
    _ref = None

    def __init__(self, app=None):
        self._models = {}
        for name, model in models.iter_models_items():
            if isinstance(model, type) and issubclass(model, AutoPath):
                self._models[model.__name__] = model
        self.routes = defaultdict(dict)


    def _init(self):
        for name, model in self._models.items():
            model.init(self)
        return self

    def add(self, registerd_route, override=True):
        print(registerd_route)
        for method in registerd_route.route.method:
            path = registerd_route.route.path.get_path()
            if method in self.routes[path]:
                if not override:
                    raise ValueError(f"La route {path} est déja défini avec {method}")
            self.routes[path][method] = registerd_route

    @classmethod
    def get_instance(cls):
        if cls._ref is None:
            cls._ref = AutoPathManager()
            cls._ref._init()
        return cls._ref

    def get_pathes(self):
        pathes = []
        for route, methods in self.routes.items():
            def wrapper(req : HttpRequest, *args, methods=methods, **kwargs):
                method = req.method
                if method not in methods:
                    return response.serv_json_not_found(f"Method {method} on {req.path}")

                if "bind_user" in methods[method].options:
                    kwargs["user"] = req.user

                ret = methods[method](req, *args, **kwargs)
                if isinstance(ret, HttpResponse):
                    return ret
                if isinstance(ret, errors.CustomException):
                    return ret.get_error()
                if isinstance(ret, Exception):
                    return response.serv_json(500, 500, err.__class__.__name__, str(err))

                return response.serv_json_ok(ret)

            pathes.append(path(route, wrapper))
        return pathes





class AutoPath:

    _path_ = None
    _key_ = "id"
    _registered_ = []
    _inited = False
    _bind_user_ = False


    def _auto_path_update(self, req, **kwargs):
        ret = self.update(json.loads(req.body))
        self.save()
        return ret

    def _auto_path_get(self, req, **kwargs):
        return self

    def _auto_path_list(self, req, **kwargs):
        if "user" in kwargs and hasattr(self, "user"):
            return self.filter(user=kwargs["user"])

        return self.all()

    def _auto_path_delete(self, req, **kwargs):
        self.delete()
        return True

    def _auto_path_create(self, req, **kwargs):
        try:
            return self.create(**json.loads(req.body), **kwargs)
        except Exception as err:
            return err

    @classmethod
    def init(cls, manager):
        if cls._inited: return
        route_options = {}
        if cls._bind_user_:
            route_options["bind_user"] = True


        root = cls._path_
        if root is None:
            root = PointPath()

        root = root / ConstPath(cls.__name__.lower()) / AttributePath(cls, cls._key_)

        manager.add(RegisteredRoute(Post(root, **route_options), cls._auto_path_update))
        manager.add(RegisteredRoute(Delete(root, **route_options), cls._auto_path_delete))
        manager.add(RegisteredRoute(Get(root, **route_options), cls._auto_path_get))

        for k in dir(cls):
            v = getattr(cls, k)
            if isinstance(v, RegisteredRoute) and v != root:
                manager.add(Route(root) / v)

        if isinstance(root.liste[-1], AttributePath):
            _root = UrlPath()
            _root.liste = root.liste[:-1]
            _root = _root / ModelPath(cls)

        manager.add(RegisteredRoute(Get(_root, **route_options), cls._auto_path_list, True))
        manager.add(RegisteredRoute(Put(_root, **route_options), cls._auto_path_create, True))

        for k in  dir(cls.objects):
            v = getattr(cls.objects, k)
            if isinstance(v, RegisteredRoute) and v != _root:
                rr = Route(_root) / v
                rr.is_manager = True
                manager.add(rr)

        cls._inited = True


